<?php
?>
<?php
namespace Joomla\Plugin\Task\Hikashop_report\Extension;

use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Factory;
use Joomla\Component\Scheduler\Administrator\Event\ExecuteTaskEvent;
use Joomla\Component\Scheduler\Administrator\Task\Status as TaskStatus;
use Joomla\Component\Scheduler\Administrator\Traits\TaskPluginTrait;
use Joomla\Event\DispatcherInterface;
use Joomla\Event\SubscriberInterface;
use Joomla\Database\DatabaseAwareTrait;
use Joomla\Database\ParameterType;
use Joomla\Filesystem\File;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Uri\Uri;
use LogicException;

\defined('_JEXEC') or die;

final class Hikashop_report extends CMSPlugin implements SubscriberInterface
{
    use TaskPluginTrait;
	use DatabaseAwareTrait;

    protected const TASKS_MAP = [
        'Hikashop_report.hikashop_report' => [
            'langConstPrefix' => 'PLG_TASK_HIKASHOP_REPORT_HIKASHOP_REPORT',
            'form'            => 'hikashop_report',
            'method'          => 'hikashop_report',
        ],
    ];

    public static function getSubscribedEvents(): array
    {
        return [
            'onTaskOptionsList' => 'advertiseRoutines',
            'onExecuteTask' => 'standardRoutineHandler',
            'onContentPrepareForm' => 'enhanceTaskItemForm',
        ];
    }

    protected $autoloadLanguage = true;

    public function __construct(DispatcherInterface $dispatcher, array $config)
    {
        parent::__construct($dispatcher, $config);
    }

    protected function hikashop_report(ExecuteTaskEvent $event): int
    {
        $this->logTask('Launching HikaShop Report task...', 'info');
		$db = Factory::getDbo();

		$params = $event->getArgument('params');

        if (!$db) {
            $this->logTask('No DB connection', 'warning');
            return TaskStatus::NO_RUN;
        }

        if (empty($params)) {
            $this->logTask('Params are empty', 'warning');
            return TaskStatus::NO_RUN;
        }

        if (empty($params->report)) {
            $this->logTask('Report not selected', 'warning');
            return TaskStatus::NO_RUN;
        }

        $report = (int)$params->report;
        $email_address = (string)$params->email_address;

        $this->logTask('Loading HikaShop', 'info');
        if (!defined('DS'))
            define('DS', DIRECTORY_SEPARATOR);
        if(!include_once(rtrim(JPATH_ADMINISTRATOR,DS).DS.'components'.DS.'com_hikashop'.DS.'helpers'.DS.'helper.php')) {
            $this->logTask('HikaShop is missing', 'warning');
            return TaskStatus::NO_RUN;
        }
        if(!hikashop_level(2)){
            $this->logTask('HikaShop Business is required', 'warning');
            return TaskStatus::NO_RUN;
        }
        $config = hikashop_config();
        if(version_compare($config->get('version', '1.0'), '5.1.0', '<')) {
            $this->logTask('HikaShop 5.1.0 minimum is required', 'warning');
            return TaskStatus::NO_RUN;
        }
        $this->logTask('Generating CSV', 'info');
        $widgetClass = hikashop_get('class.widget');
        $content = $widgetClass->csv($report, 'return');
        $path = 'plugins/task/hikashop_report/files/'.$widgetClass->filename;

        $this->logTask('Writing CSV file', 'info');
        File::write(HIKASHOP_ROOT.$path, $content);


        if (!empty($params->email_address)) {
            $this->logTask('Loading email', 'info');
            $mailClass = hikashop_get('class.mail');
            $content = array('type' => 'csv_export');
            $mail = $mailClass->get('massaction_notification',$content);
            $mail->subject = Text::_('HIKASHOP_REPORT_EMAIL_SUBJECT');
            $mail->html = '1';
            $csv = new \stdClass();
            $csv->name = basename($path);
            $csv->filename = HIKASHOP_ROOT.$path;
            $csv->url = Uri::base(true).$path;
            $mail->attachments = array($csv);
            $mail->dst_name = '';
            $mail->dst_email = explode(',',$email_address);
            $this->logTask('Sending email', 'info');
            try {
                ob_start();
                $mailClass->sendMail($mail);
                ob_clean();
            }catch (\Exception $e){
                $this->logTask('Email error: '.$e->getMessage(), 'warning');
                return TaskStatus::NO_RUN;
            }
        }


        $this->logTask('Completed processing HikaShop Report.', 'info');

        return TaskStatus::OK;
    }
}
